### ====================================================================
###  @Awk-file{
###     author          = "Nelson H. F. Beebe",
###     version         = "3.03",
###     date            = "09 February 2005",
###     time            = "16:16:33 MST",
###     filename        = "tgrindex.awk",
###     address         = "University of Utah
###                        Department of Mathematics, 110 LCB
###                        155 S 1400 E RM 233
###                        Salt Lake City, UT 84112-0090
###                        USA",
###     telephone       = "+1 801 581 5254",
###     FAX             = "+1 801 581 4148",
###     URL             = "http://www.math.utah.edu/~beebe",
###     checksum        = "47513 143 530 4269",
###     email           = "beebe@math.utah.edu, beebe@acm.org,
###                        beebe@computer.org (Internet)",
###     codetable       = "ISO/ASCII",
###     keywords        = "index, multi-column, program listings, TeX,
###                       tgrind, typesetting",
###     supported       = "yes",
###     docstring       = "Filter a TeX file output by tgrind to produce
###                        sorted multi-column indexes of procedure
###                        names, one index for each \File{},{},{} line.
###                        This program is used by the companion UNIX
###                        shell script, tgrind.sh, and possibly by
###                        scripts or programs for other operating
###                        system environments.
###
###                        Usage:
###                            nawk -f tgrindex.awk <TeX-file >new-TeX-file
###
###                        The checksum field above contains a CRC-16
###                        checksum as the first value, followed by the
###                        equivalent of the standard UNIX wc (word
###                        count) utility output of lines, words, and
###                        characters.  This is produced by Robert
###                        Solovay's checksum utility.",
###  }
### ====================================================================

BEGIN {
    # Seed the pseudo-random number generator with the time-of-day so
    # that we can create a highly random index file name to avoid
    # collisions.  Filenames are no more than 8+1+3 characters long, to
    # meet worst case limitations of PC DOS.  The temporary file is
    # created in the current directory.
    srand()
    index_file = sprintf("tg_%05d.idx", int(rand() * 100000))
    # old style sort
    # sortpipe = "sort -f +0 -1 +1n -2 >" index_file
    # new POSIX style
    sortpipe = "sort -f -k1,1 -k2,2n -2 >" index_file
}

/\\L\{/ \
{
    line_number++
}

/\\Proc\{[^}]*\}/ \
{
    match($0,/\\Proc\{[^}]*\}/)
    print substr($0,RSTART+6,RLENGTH-7), line_number | sortpipe
}

/^\\TgrindFile\{.*\}\{.*\}\{.*\}/ \
{
    new_file()
    next
}

/\\EndTgrind/ \
{				# end of file processing
    new_file()
    next
}

# Copy all other lines verbatim
{
    print $0
}

function lc(c, k)
{				# return the lower-case equivalent of c
    # gawk, and some newer nawks, have tolower(), but since not all do,
    # we provide our own equivalent, under a different name to avoid conflicts,
    # and restricted to single characters, rather than strings.
    k = index("ABCDEFGHIJKLMNOPQRSTUVWXYZ", c)
    return (k > 0) ? substr("abcdefghijklmnopqrstuvwxyz",k,1) : c
}

function new_file( line)
{
    line = $0
    if (line_number > 0)
    {
	write_index()
	line_number = 0
    }
    print line
}

function write_entry( k)
{
    if ((first_letter != "") && (lc(substr(last1,1,1)) != first_letter))
	print "    \\NewIndexLetter"
    printf("\t\\Entry{%s}{%s}\n", last1, last2)
    first_letter = lc(substr(last1,1,1))
}

function write_index()
{
    close(sortpipe)

    n = 0
    last1 = ""
    last2 = ""
    while ((getline < index_file) > 0)
    {
	n++
	if (n == 1)
	    print "\\BeginIndex"
	if ($1 == last1)	# same procedure, so append line numbers
	    last2 = last2 ", " $2
	else			# new procedure
	{
	    if (last1 != "")
		write_entry()
	    last1 = $1
	    last2 = $2
	}
    }
    if (n > 0)
    {
	write_entry()
	print "\\EndIndex"
    }

    close(index_file)
    system("/bin/rm " index_file " 2>/dev/null")
}
